/*
  For more information, please see: http://software.sci.utah.edu

  The MIT License

  Copyright (c) 2004 Scientific Computing and Imaging Institute,
  University of Utah.

  License for the specific language governing rights and limitations under
  Permission is hereby granted, free of charge, to any person obtaining a
  copy of this software and associated documentation files (the "Software"),
  to deal in the Software without restriction, including without limitation
  the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included
  in all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
  THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.
*/


/*
 *  TxtConnection.h: 
 *
 *  Written by:
 *   Keming Zhang
 *   Department of Computer Science
 *   University of Utah
 *   September 2005
 *
 */

#ifndef CCA_Components_TxtBuilder_TxtConnection_h
#define CCA_Components_TxtBuilder_TxtConnection_h

#include <curses.h>

namespace SCIRun {

  class Point{
  public:
    Point(){};
    Point(int x, int y):x(x), y(y){}
  public:
    int x, y;
  };

  class Rect{
  public:
    Rect(){};
    Rect(int x, int y, int w, int h):x(x), y(y), w(w), h(h){}
    int top(){
      return y;
    };
    int bottom(){
      return y+h-1;
    };
    Point upoint(){
      return Point(x+w, y+h/2);
    };
    Point ppoint(){
      return Point(x, y+h/2);
    };
  public:
    int x, y, w, h;
  };


  class TxtConnection
    {
    public:
      TxtConnection(WINDOW *win_main);
      void resetPoints(Rect rUse, Rect rProvide);
      void draw();
      void erase();
    protected:
      enum TxtDirection{N, U, D, L, R}; 
      int cline(TxtDirection d1,TxtDirection d2);
      WINDOW *win;
      int n_pts;
      Point pa[6];
    };

} //namespace SCIRun

#endif
