/*
   For more information, please see: http://software.sci.utah.edu

   The MIT License

   Copyright (c) 2004 Scientific Computing and Imaging Institute,
   University of Utah.

   License for the specific language governing rights and limitations under
   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included
   in all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
   THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
   DEALINGS IN THE SOFTWARE.
*/


/*
 *  Viewer.cc:
 *
 *  Written by:
 *   Keming Zhang
 *   Department of Computer Science
 *   University of Utah
 *   May 2002
 *
 */

#include <CCA/Components/Viewer/Viewer.h>
#include <CCA/Components/Viewer/ViewerDialog.h>
#include <Core/CCA/datawrapper/vector2d.h>

#include <math.h>
#include <stdlib.h>
#include <iostream>
#include <fstream>


namespace Viewer {

using namespace SCIRun;

extern "C" sci::cca::Component::pointer make_SCIRun_Viewer()
{
  return sci::cca::Component::pointer(new Viewer());
}

Viewer::Viewer()
{
}

Viewer::~Viewer()
{
  services->removeProvidesPort("viewer");
}

void Viewer::setServices(const sci::cca::Services::pointer& svc)
{
  services = svc;
  services->addProvidesPort(ViewPort::pointer(new ViewPort), "viewer",
                            "sci.cca.ports.ViewPort", sci::cca::TypeMap::pointer(0));

//   ViewerUIPort *uip = new ViewerUIPort();
//   uip->setParent(this);
//   ttUIPort::pointer uiPortPtr(uip);
//   svc->addProvidesPort(uiPortPtr, "ui", "sci.cca.ports.UIPort", sci::cca::TypeMap::pointer(0));
}

int
ViewPort::view2dPDE(const SSIDL::array1<double> &nodes,
                    const SSIDL::array1<int> &triangles,
                    const SSIDL::array1<double> &solution)
{
  if (nodes.size() / 2 != solution.size()) {
    wxMessageBox(wxT("Mesh and Field do not match!"), wxT("Viewer"), wxOK|wxICON_ERROR, 0);
    return 1;
  }

  ViewerDialog vd(0, nodes, triangles, solution);
  vd.ShowModal();

  return 0;
}

}
