/*
   For more information, please see: http://software.sci.utah.edu

   The MIT License

   Copyright (c) 2004 Scientific Computing and Imaging Institute,
   University of Utah.

   License for the specific language governing rights and limitations under
   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included
   in all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
   THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
   DEALINGS IN THE SOFTWARE.
*/


/*
 *  BabelComponentInstance.h:
 *
 *  Written by:
 *   Keming Zhang
 *   Department of Computer Science
 *   University of Utah
 *   July 2002
 *
 */
#ifndef Framework_Framework_BabelComponentInstance_h
#define Framework_Framework_BabelComponentInstance_h

#include <Framework/Babel/framework.hxx>
#include <Framework/Babel/gov_cca.hxx>

#include <Framework/Babel/BabelPortInstance.h>
#include <Framework/ComponentInstance.h>
#include <Framework/PortInstanceIterator.h>
#include <Core/CCA/spec/cca_sidl.h>

#include <map>
#include <string>

namespace SCIRun {

/** \class BabelCCAGoPort
 *
 * Intermediary between a Babel component model GoPort and a CCA component model GoPort.
 */
class BabelCCAGoPort : public virtual sci::cca::ports::GoPort
{
public:
  BabelCCAGoPort(const UCXX ::gov::cca::ports::GoPort &port);
  virtual int go();

private:
  UCXX ::gov::cca::ports::GoPort port;
};

/** \class BabelCCAUIPort
 *
 * Intermediary between a Babel component model UIPort and a CCA component model UIPort.
 */
class BabelCCAUIPort : public virtual sci::cca::ports::UIPort
{
public:
  BabelCCAUIPort(const UCXX ::gov::cca::ports::UIPort &port);
  virtual int ui();
private:
  UCXX ::gov::cca::ports::UIPort port;
};


class BabelPortInstance;
class Services;

/** \class BabelComponentInstance
 *
 * A handle to an instantiation of a Babel component in the SCIRun framework.  This
 * class is a container for information necessary to interact with an
 * instantiation of a framework component such as its unique instance name, its
 * type, its ports, and the framework to which it belongs.
 */
class BabelComponentInstance : public ComponentInstance
{
public:
  BabelComponentInstance(SCIRunFramework* framework,
			 const std::string& instanceName,
			 const std::string& className,
			 const UCXX ::gov::cca::TypeMap& typemap,
			 const UCXX ::gov::cca::Component& component,
			 const UCXX ::framework::Services& svc);
  virtual ~BabelComponentInstance();

  /** A proxy method for gov::cca::Services.  Calls the corresponding method in
      SCIRunFramework::Services. */
  UCXX ::gov::cca::Port getPort(const std::string& name);


  /** A proxy method for gov::cca::Services.  Calls the corresponding method in
      SCIRunFramework::Services. */
  UCXX ::gov::cca::Port getPortNonblocking(const std::string& name);


  /** A proxy method for gov::cca::Services.  Calls the corresponding method in
      SCIRunFramework::Services. */
  void releasePort(const std::string& name);

  /** A proxy method for gov::cca::Services.  Calls the corresponding method in
      SCIRunFramework::Services. */
  UCXX ::gov::cca::TypeMap createTypeMap();


  /** A proxy method for gov::cca::Services.  Calls the corresponding method in
      SCIRunFramework::Services. */
  void registerUsesPort(const std::string& name, const std::string& type,
			const UCXX ::gov::cca::TypeMap& properties);


  /** A proxy method for gov::cca::Services.  Calls the corresponding method in
      SCIRunFramework::Services. */
  void unregisterUsesPort(const std::string& name);

  /** A proxy method for gov::cca::Services.  Calls the corresponding method in
      SCIRunFramework::Services. */
  void addProvidesPort(const UCXX ::gov::cca::Port& port,
		       const std::string& name,
		       const std::string& type,
		       const UCXX ::gov::cca::TypeMap& properties);

  void removeProvidesPort(const std::string& name);

  /** A proxy method for gov::cca::Services.  Calls the corresponding method in
      SCIRunFramework::Services. */
  UCXX ::gov::cca::ComponentID getComponentID();

  /** ? */
  virtual PortInstance* getPortInstance(const std::string& name);

  /** ? */
  virtual PortInstanceIterator* getPorts();


  /** A proxy method for gov::cca::Services.  Calls the corresponding method in
      SCIRunFramework::Services. */
//   UCXX ::gov::cca::TypeMap getPortProperties(const std::string& portName);
  // these do nothing at the moment - implement after compiler changeover
  virtual sci::cca::TypeMap::pointer getPortProperties(const std::string& /*portName*/);
  virtual void setPortProperties(const std::string& /*portName*/, const sci::cca::TypeMap::pointer& /*tm*/) {}

private:
  UCXX ::framework::Services svc;

  /** ? */
  class Iterator : public PortInstanceIterator
  {
    PortInstanceMap *ports;
    PortInstanceMap::iterator iter;
  public:
    Iterator(BabelComponentInstance*);
    virtual ~Iterator();
    virtual PortInstance* get();
    virtual bool done();
    virtual void next();
  private:
    Iterator(const Iterator&);
    Iterator& operator=(const Iterator&);
    //sci::cca::ComponentID::pointer cid;
  };
  UCXX ::gov::cca::Component component;

  BabelComponentInstance(const BabelComponentInstance&);
  BabelComponentInstance& operator=(const BabelComponentInstance&);
};

} // end namespace SCIRun

#endif
