/*
   For more information, please see: http://software.sci.utah.edu

   The MIT License

   Copyright (c) 2004 Scientific Computing and Imaging Institute,
   University of Utah.

   License for the specific language governing rights and limitations under
   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included
   in all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
   THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
   DEALINGS IN THE SOFTWARE.
*/


/*
 *  BabelPortInstance.h:
 *
 *  Written by:
 *   Keming Zhang
 *   Department of Computer Science
 *   University of Utah
 *   July 2002
 *
 */

#ifndef Framework_Babel_BabelPortInstance_h
#define Framework_Babel_BabelPortInstance_h

#include <Framework/Babel/gov_cca.hxx>

#include <Framework/PortInstance.h>
#include <Core/Thread/Mutex.h>
#include <Core/Thread/Guard.h>
#include <Core/CCA/spec/cca_sidl.h>

#include <map>
#include <string>
#include <vector>

namespace SCIRun {

/**
 * \class BabelPortInstance
 *
 *
 */
class BabelPortInstance : public PortInstance
{
public:
  BabelPortInstance(const std::string& portname, const std::string& classname,
		    const UCXX ::gov::cca::TypeMap& properties,
		    PortType porttype);
  BabelPortInstance(const std::string& portname, const std::string& classname,
		    const UCXX ::gov::cca::TypeMap& properties,
		    const UCXX ::gov::cca::Port& port,
		    PortType porttype);
  virtual ~BabelPortInstance();
  virtual bool connect(PortInstance*);
  virtual PortInstance::PortType portType();
  virtual std::string getType();
  virtual std::string getModel();
  virtual std::string getUniqueName();
  virtual bool disconnect(PortInstance*);
  virtual bool canConnectTo(PortInstance*);
  virtual bool available();
  virtual PortInstance* getPeer();

  std::string getName();
  int getConnectionCount() { return connections.size(); }

  void incrementUseCount();
  bool decrementUseCount();

  /** Test use counter. */
  bool portInUse();

  const UCXX ::gov::cca::Port& getPort() { return port; }
  //UCXX ::gov::cca::TypeMap getProperties();

private:
  //friend class BabelComponentInstance;
  PortType porttype;
  std::vector<PortInstance*> connections;
  SCIRun::Mutex lock_connections;

  std::string name;
  std::string type;
  UCXX ::gov::cca::TypeMap properties;
  UCXX ::gov::cca::Port port;

  int useCount;

  BabelPortInstance(const BabelPortInstance&);
  BabelPortInstance& operator=(const BabelPortInstance&);

  // this will have to be updated
  virtual sci::cca::TypeMap::pointer getProperties() { return sci::cca::TypeMap::pointer(0); }
  virtual void setProperties(const sci::cca::TypeMap::pointer& tm) {}
};

} //namespace SCIRun

#endif
