/* $Id: resourceseed.h 19308 2003-07-22 21:00:00Z simpson $ */


/*
 * Copyright  2000 The Regents of the University of California. 
 * All Rights Reserved. 
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for educational, research and non-profit purposes, without
 * fee, and without a written agreement is hereby granted, provided that the
 * above copyright notice, this paragraph and the following three paragraphs
 * appear in all copies. 
 *
 * Permission to incorporate this software into commercial products may be
 * obtained by contacting
 * Eric Lund
 * Technology Transfer Office 
 * 9500 Gilman Drive 
 * 411 University Center 
 * University of California 
 * La Jolla, CA 92093-0093
 * (858) 534-0175
 * ericlund@ucsd.edu
 *
 * This software program and documentation are copyrighted by The Regents of
 * the University of California. The software program and documentation are
 * supplied "as is", without any accompanying services from The Regents. The
 * Regents does not warrant that the operation of the program will be
 * uninterrupted or error-free. The end-user understands that the program was
 * developed for research purposes and is advised not to rely exclusively on
 * the program for any reason. 
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
 * LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION,
 * EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED
 * HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO
 * OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
 * MODIFICATIONS. 
 */


#ifndef RESOURCESEED_H
#define RESOURCESEED_H


/*
 * This package provides facilities for measuring system resource usage and
 * availability.
 */


#include <sys/types.h> /* pid_t */


#ifdef __cplusplus
extern "C" {
#endif


/*
 * Measures the amount of CPU time available to a new process.  Returns the
 * percentage of CPU (between 0 and 100) available to a new nice #nice#
 * processes.
 */
double
ASRES_FreeCpu(int nice);


/* Returns the bytes available on the disk that contains #path#. */
double
ASRES_FreeDisk(const char *path);


/* Returns the amount of real memory available to user processes. */
double
ASRES_FreeMemory(void);


/* Returns the number of CPU seconds used by #process# since it started. */
double
ASRES_ProcessCpuTime(pid_t process);


#ifdef ASRES_SHORT_NAMES

#define FreeCpu ASRES_FreeCpu
#define FreeDisk ASRES_FreeDisk
#define FreeMemory ASRES_FreeMemory
#define ProcessCpuTime ASRES_ProcessCpuTime

#endif


#ifdef __cplusplus  
}  
#endif


#endif
